/* ***********************************************************
 * This file was automatically generated on 2012-12-20.      *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generator git on tinkerforge.com                   *
 *************************************************************/

using System;

namespace Tinkerforge
{
	/// <summary>
	///  Device for controlling DC motors
	/// </summary>
	public class BrickDC : Device 
	{
		private static byte FUNCTION_SET_VELOCITY = 1;
		private static byte FUNCTION_GET_VELOCITY = 2;
		private static byte FUNCTION_GET_CURRENT_VELOCITY = 3;
		private static byte FUNCTION_SET_ACCELERATION = 4;
		private static byte FUNCTION_GET_ACCELERATION = 5;
		private static byte FUNCTION_SET_PWM_FREQUENCY = 6;
		private static byte FUNCTION_GET_PWM_FREQUENCY = 7;
		private static byte FUNCTION_FULL_BRAKE = 8;
		private static byte FUNCTION_GET_STACK_INPUT_VOLTAGE = 9;
		private static byte FUNCTION_GET_EXTERNAL_INPUT_VOLTAGE = 10;
		private static byte FUNCTION_GET_CURRENT_CONSUMPTION = 11;
		private static byte FUNCTION_ENABLE = 12;
		private static byte FUNCTION_DISABLE = 13;
		private static byte FUNCTION_IS_ENABLED = 14;
		private static byte FUNCTION_SET_MINIMUM_VOLTAGE = 15;
		private static byte FUNCTION_GET_MINIMUM_VOLTAGE = 16;
		private static byte FUNCTION_SET_DRIVE_MODE = 17;
		private static byte FUNCTION_GET_DRIVE_MODE = 18;
		private static byte FUNCTION_SET_CURRENT_VELOCITY_PERIOD = 19;
		private static byte FUNCTION_GET_CURRENT_VELOCITY_PERIOD = 20;
		private static byte CALLBACK_UNDER_VOLTAGE = 21;
		private static byte CALLBACK_EMERGENCY_SHUTDOWN = 22;
		private static byte CALLBACK_VELOCITY_REACHED = 23;
		private static byte CALLBACK_CURRENT_VELOCITY = 24;
		private static byte FUNCTION_RESET = 243;
		private static byte FUNCTION_GET_CHIP_TEMPERATURE = 242;


		/// <summary>
		///  This callback is triggered when the input voltage drops below the value set by
		///  <see cref="Tinkerforge.BrickDC.SetMinimumVoltage"/>. The parameter is the current voltage given
		///  in mV.
		/// </summary>
		public delegate void UnderVoltage(ushort voltage);

		/// <summary>
		///  This callback is triggered if either the current consumption
		///  is too high (above 5A) or the temperature of the driver chip is too high
		///  (above 175°C). These two possibilities are essentially the same, since the
		///  temperature will reach this threshold immediately if the motor consumes too
		///  much current. In case of a voltage below 3.3V (external or stack) this
		///  callback is triggered as well.
		///  
		///  If this callback is triggered, the driver chip gets disabled at the same time.
		///  That means, <see cref="Tinkerforge.BrickDC.Enable"/> has to be called to drive the motor again.
		///  
		///  <note>
		///   This callback only works in Drive/Brake mode (see <see cref="Tinkerforge.BrickDC.SetDriveMode"/>). In
		///   Drive/Coast mode it is unfortunately impossible to reliably read the
		///   overcurrent/overtemperature signal from the driver chip.
		///  </note>
		/// </summary>
		public delegate void EmergencyShutdown();

		/// <summary>
		///  This callback is triggered whenever a set velocity is reached. For example:
		///  If a velocity of 0 is present, acceleration is set to 5000 and velocity
		///  to 10000, <see cref="Tinkerforge.BrickDC.VelocityReached"/> will be triggered after about 2 seconds, when
		///  the set velocity is actually reached.
		///  
		///  <note>
		///   Since we can't get any feedback from the DC motor, this only works if the
		///   acceleration (see <see cref="Tinkerforge.BrickDC.SetAcceleration"/>) is set smaller or equal to the
		///   maximum acceleration of the motor. Otherwise the motor will lag behind the
		///   control value and the callback will be triggered too early.
		///  </note>
		/// </summary>
		public delegate void VelocityReached(short velocity);

		/// <summary>
		///  This callback is triggered with the period that is set by
		///  <see cref="Tinkerforge.BrickDC.SetCurrentVelocityPeriod"/>. The parameter is the *current* velocity
		///  used by the motor.
		///  
		///  <see cref="Tinkerforge.BrickDC.CurrentVelocity"/> is only triggered after the set period if there is
		///  a change in the velocity.
		/// </summary>
		public delegate void CurrentVelocity(short velocity);

		/// <summary>
		///  Creates an object with the unique device ID <c>uid</c>. This object can
		///  then be added to the IP connection.
		/// </summary>
		public BrickDC(string uid) : base(uid) 
		{
			this.expectedName = "DC Brick";
			this.bindingVersion[0] = 1;
			this.bindingVersion[1] = 0;
			this.bindingVersion[2] = 1;
			messageCallbacks[CALLBACK_UNDER_VOLTAGE] = new MessageCallback(CallbackUnderVoltage);
			messageCallbacks[CALLBACK_EMERGENCY_SHUTDOWN] = new MessageCallback(CallbackEmergencyShutdown);
			messageCallbacks[CALLBACK_VELOCITY_REACHED] = new MessageCallback(CallbackVelocityReached);
			messageCallbacks[CALLBACK_CURRENT_VELOCITY] = new MessageCallback(CallbackCurrentVelocity);
		}

		/// <summary>
		///  Sets the velocity of the motor. Whereas -32767 is full speed backward,
		///  0 is stop and 32767 is full speed forward. Depending on the
		///  acceleration (see <see cref="Tinkerforge.BrickDC.SetAcceleration"/>), the motor is not immediately
		///  brought to the velocity but smoothly accelerated.
		///  
		///  The velocity describes the duty cycle of the PWM with which the motor is
		///  controlled, e.g. a velocity of 3277 sets a PWM with a 10% duty cycle.
		///  You can not only control the duty cycle of the PWM but also the frequency,
		///  see <see cref="Tinkerforge.BrickDC.SetPWMFrequency"/>.
		///  
		///  The default velocity is 0.
		/// </summary>
		public void SetVelocity(short velocity)
		{
			byte[] data_ = new byte[6];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_VELOCITY, 1, data_);
			LEConverter.To((ushort)6, 2, data_);
			LEConverter.To(velocity, 4, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns the velocity as set by <see cref="Tinkerforge.BrickDC.SetVelocity"/>.
		/// </summary>
		public short GetVelocity()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_VELOCITY, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_VELOCITY, out response);

			return LEConverter.ShortFrom(4, response);
		}

		/// <summary>
		///  Returns the *current* velocity of the motor. This value is different
		///  from <see cref="Tinkerforge.BrickDC.GetVelocity"/> whenever the motor is currently accelerating
		///  to a goal set by <see cref="Tinkerforge.BrickDC.SetVelocity"/>.
		/// </summary>
		public short GetCurrentVelocity()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_CURRENT_VELOCITY, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_CURRENT_VELOCITY, out response);

			return LEConverter.ShortFrom(4, response);
		}

		/// <summary>
		///  Sets the acceleration of the motor. It is given in *velocity/s*. An
		///  acceleration of 10000 means, that every second the velocity is increased
		///  by 10000 (or about 30% duty cycle).
		///  
		///  For example: If the current velocity is 0 and you want to accelerate to a
		///  velocity of 16000 (about 50% duty cycle) in 10 seconds, you should set
		///  an acceleration of 1600.
		///  
		///  If acceleration is set to 0, there is no speed ramping, i.e. a new velocity
		///  is immediately given to the motor.
		///  
		///  The default acceleration is 10000.
		/// </summary>
		public void SetAcceleration(ushort acceleration)
		{
			byte[] data_ = new byte[6];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_ACCELERATION, 1, data_);
			LEConverter.To((ushort)6, 2, data_);
			LEConverter.To(acceleration, 4, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns the acceleration as set by <see cref="Tinkerforge.BrickDC.SetAcceleration"/>.
		/// </summary>
		public ushort GetAcceleration()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_ACCELERATION, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_ACCELERATION, out response);

			return LEConverter.UShortFrom(4, response);
		}

		/// <summary>
		///  Sets the frequency (in Hz) of the PWM with which the motor is driven.
		///  The possible range of the frequency is 1-20000Hz. Often a high frequency
		///  is less noisy and the motor runs smoother. However, with a low frequency
		///  there are less switches and therefore fewer switching losses. Also with
		///  most motors lower frequencies enable higher torque.
		///  
		///  If you have no idea what all this means, just ignore this function and use
		///  the default frequency, it will very likely work fine.
		///  
		///  The default frequency is 15 kHz.
		/// </summary>
		public void SetPWMFrequency(ushort frequency)
		{
			byte[] data_ = new byte[6];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_PWM_FREQUENCY, 1, data_);
			LEConverter.To((ushort)6, 2, data_);
			LEConverter.To(frequency, 4, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns the PWM frequency (in Hz) as set by <see cref="Tinkerforge.BrickDC.SetPWMFrequency"/>.
		/// </summary>
		public ushort GetPWMFrequency()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_PWM_FREQUENCY, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_PWM_FREQUENCY, out response);

			return LEConverter.UShortFrom(4, response);
		}

		/// <summary>
		///  Executes an active full brake.
		///  
		///  <note type="caution">
		///   This function is for emergency purposes,
		///   where an immediate brake is necessary. Depending on the current velocity and
		///   the strength of the motor, a full brake can be quite violent.
		///  </note>
		///  
		///  Call <see cref="Tinkerforge.BrickDC.SetVelocity"/> with 0 if you just want to stop the motor.
		/// </summary>
		public void FullBrake()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_FULL_BRAKE, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns the stack input voltage in mV. The stack input voltage is the
		///  voltage that is supplied via the stack, i.e. it is given by a
		///  Step-Down or Step-Up Power Supply.
		/// </summary>
		public ushort GetStackInputVoltage()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_STACK_INPUT_VOLTAGE, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_STACK_INPUT_VOLTAGE, out response);

			return LEConverter.UShortFrom(4, response);
		}

		/// <summary>
		///  Returns the external input voltage in mV. The external input voltage is
		///  given via the black power input connector on the DC Brick.
		///  
		///  If there is an external input voltage and a stack input voltage, the motor
		///  will be driven by the external input voltage. If there is only a stack
		///  voltage present, the motor will be driven by this voltage.
		///  
		///  <note type="caution">
		///   This means, if you have a high stack voltage and a low external voltage,
		///   the motor will be driven with the low external voltage. If you then remove
		///   the external connection, it will immediately be driven by the high
		///   stack voltage.
		///  </note>
		/// </summary>
		public ushort GetExternalInputVoltage()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_EXTERNAL_INPUT_VOLTAGE, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_EXTERNAL_INPUT_VOLTAGE, out response);

			return LEConverter.UShortFrom(4, response);
		}

		/// <summary>
		///  Returns the current consumption of the motor in mA.
		/// </summary>
		public ushort GetCurrentConsumption()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_CURRENT_CONSUMPTION, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_CURRENT_CONSUMPTION, out response);

			return LEConverter.UShortFrom(4, response);
		}

		/// <summary>
		///  Enables the driver chip. The driver parameters can be configured (velocity,
		///  acceleration, etc) before it is enabled.
		/// </summary>
		public void Enable()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_ENABLE, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Disables the driver chip. The configurations are kept (velocity,
		///  acceleration, etc) but the motor is not driven until it is enabled again.
		/// </summary>
		public void Disable()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_DISABLE, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns *true* if the driver chip is enabled, *false* otherwise.
		/// </summary>
		public bool IsEnabled()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_IS_ENABLED, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_IS_ENABLED, out response);

			return LEConverter.BoolFrom(4, response);
		}

		/// <summary>
		///  Sets the minimum voltage in mV, below which the <see cref="Tinkerforge.BrickDC.UnderVoltage"/> callback
		///  is triggered. The minimum possible value that works with the DC Brick is 5V.
		///  You can use this function to detect the discharge of a battery that is used
		///  to drive the motor. If you have a fixed power supply, you likely do not need
		///  this functionality.
		///  
		///  The default value is 5V.
		/// </summary>
		public void SetMinimumVoltage(ushort voltage)
		{
			byte[] data_ = new byte[6];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_MINIMUM_VOLTAGE, 1, data_);
			LEConverter.To((ushort)6, 2, data_);
			LEConverter.To(voltage, 4, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns the minimum voltage as set by <see cref="Tinkerforge.BrickDC.SetMinimumVoltage"/>
		/// </summary>
		public ushort GetMinimumVoltage()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_MINIMUM_VOLTAGE, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_MINIMUM_VOLTAGE, out response);

			return LEConverter.UShortFrom(4, response);
		}

		/// <summary>
		///  Sets the drive mode. Possible modes are:
		///  
		///  * 0 = Drive/Brake
		///  * 1 = Drive/Coast
		///  
		///  These modes are different kinds of motor controls.
		///  
		///  In Drive/Brake mode, the motor is always either driving or braking. There
		///  is no freewheeling. Advantages are: A more linear correlation between
		///  PWM and velocity, more exact accelerations and the possibility to drive
		///  with slower velocities.
		///  
		///  In Drive/Coast mode, the motor is always either driving or freewheeling.
		///  Advantages are: Less current consumption and less demands on the motor and
		///  driver chip.
		///  
		///  The default value is 0 = Drive/Brake.
		/// </summary>
		public void SetDriveMode(byte mode)
		{
			byte[] data_ = new byte[5];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_DRIVE_MODE, 1, data_);
			LEConverter.To((ushort)5, 2, data_);
			LEConverter.To(mode, 4, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns the drive mode, as set by <see cref="Tinkerforge.BrickDC.SetDriveMode"/>.
		/// </summary>
		public byte GetDriveMode()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_DRIVE_MODE, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_DRIVE_MODE, out response);

			return LEConverter.ByteFrom(4, response);
		}

		/// <summary>
		///  Sets a period in ms with which the <see cref="Tinkerforge.BrickDC.CurrentVelocity"/> callback is triggered.
		///  A period of 0 turns the callback off.
		///  
		///  The default value is 0.
		/// </summary>
		public void SetCurrentVelocityPeriod(ushort period)
		{
			byte[] data_ = new byte[6];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_CURRENT_VELOCITY_PERIOD, 1, data_);
			LEConverter.To((ushort)6, 2, data_);
			LEConverter.To(period, 4, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns the period as set by <see cref="Tinkerforge.BrickDC.SetCurrentVelocityPeriod"/>.
		/// </summary>
		public ushort GetCurrentVelocityPeriod()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_CURRENT_VELOCITY_PERIOD, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_CURRENT_VELOCITY_PERIOD, out response);

			return LEConverter.UShortFrom(4, response);
		}

		/// <summary>
		///  Calling this function will reset the Brick. Calling this function
		///  on a Brick inside of a stack will reset the whole stack.
		///  
		///  After a reset you have to create new device objects,
		///  calling functions on the existing ones will result in
		///  undefined behavior!
		///  
		///  .. versionadded:: 1.1.3~(Firmware)
		/// </summary>
		public void Reset()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_RESET, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns the temperature in °C/10 as measured inside the microcontroller. The
		///  value returned is not the ambient temperature!
		///  
		///  The temperature is only proportional to the real temperature and it has an
		///  accuracy of +-15%. Practically it is only useful as an indicator for
		///  temperature changes.
		///  
		///  .. versionadded:: 1.1.3~(Firmware)
		/// </summary>
		public short GetChipTemperature()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_CHIP_TEMPERATURE, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_CHIP_TEMPERATURE, out response);

			return LEConverter.ShortFrom(4, response);
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetVelocity(out short velocity)
		{
			velocity = GetVelocity();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetCurrentVelocity(out short velocity)
		{
			velocity = GetCurrentVelocity();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetAcceleration(out ushort acceleration)
		{
			acceleration = GetAcceleration();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetPWMFrequency(out ushort frequency)
		{
			frequency = GetPWMFrequency();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetStackInputVoltage(out ushort voltage)
		{
			voltage = GetStackInputVoltage();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetExternalInputVoltage(out ushort voltage)
		{
			voltage = GetExternalInputVoltage();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetCurrentConsumption(out ushort voltage)
		{
			voltage = GetCurrentConsumption();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void IsEnabled(out bool enabled)
		{
			enabled = IsEnabled();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetMinimumVoltage(out ushort voltage)
		{
			voltage = GetMinimumVoltage();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetDriveMode(out byte mode)
		{
			mode = GetDriveMode();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetCurrentVelocityPeriod(out ushort period)
		{
			period = GetCurrentVelocityPeriod();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetChipTemperature(out short temperature)
		{
			temperature = GetChipTemperature();
		}

		internal int CallbackUnderVoltage(byte[] data_)
		{
			ushort voltage = LEConverter.UShortFrom(4, data_);

			((UnderVoltage)callbacks[CALLBACK_UNDER_VOLTAGE])(voltage);
			return 6;
		}

		internal int CallbackEmergencyShutdown(byte[] data_)
		{
			((EmergencyShutdown)callbacks[CALLBACK_EMERGENCY_SHUTDOWN])();
			return 4;
		}

		internal int CallbackVelocityReached(byte[] data_)
		{
			short velocity = LEConverter.ShortFrom(4, data_);

			((VelocityReached)callbacks[CALLBACK_VELOCITY_REACHED])(velocity);
			return 6;
		}

		internal int CallbackCurrentVelocity(byte[] data_)
		{
			short velocity = LEConverter.ShortFrom(4, data_);

			((CurrentVelocity)callbacks[CALLBACK_CURRENT_VELOCITY])(velocity);
			return 6;
		}

		/// <summary>
		///  Registers a callback function.
		/// </summary>
		public void RegisterCallback(System.Delegate d)
		{
			if(d.GetType() == typeof(UnderVoltage))
			{
				callbacks[CALLBACK_UNDER_VOLTAGE] = d;
			}
			else if(d.GetType() == typeof(EmergencyShutdown))
			{
				callbacks[CALLBACK_EMERGENCY_SHUTDOWN] = d;
			}
			else if(d.GetType() == typeof(VelocityReached))
			{
				callbacks[CALLBACK_VELOCITY_REACHED] = d;
			}
			else if(d.GetType() == typeof(CurrentVelocity))
			{
				callbacks[CALLBACK_CURRENT_VELOCITY] = d;
			}
		}
	}
}
