/* ***********************************************************
 * This file was automatically generated on 2012-12-20.      *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generator git on tinkerforge.com                   *
 *************************************************************/

using System;

namespace Tinkerforge
{
	/// <summary>
	///  Device for receiving GPS position
	/// </summary>
	public class BrickletGPS : Device 
	{
		private static byte FUNCTION_GET_COORDINATES = 1;
		private static byte FUNCTION_GET_STATUS = 2;
		private static byte FUNCTION_GET_ALTITUDE = 3;
		private static byte FUNCTION_GET_MOTION = 4;
		private static byte FUNCTION_GET_DATE_TIME = 5;
		private static byte FUNCTION_RESTART = 6;
		private static byte FUNCTION_SET_COORDINATES_CALLBACK_PERIOD = 7;
		private static byte FUNCTION_GET_COORDINATES_CALLBACK_PERIOD = 8;
		private static byte FUNCTION_SET_STATUS_CALLBACK_PERIOD = 9;
		private static byte FUNCTION_GET_STATUS_CALLBACK_PERIOD = 10;
		private static byte FUNCTION_SET_ALTITUDE_CALLBACK_PERIOD = 11;
		private static byte FUNCTION_GET_ALTITUDE_CALLBACK_PERIOD = 12;
		private static byte FUNCTION_SET_DATE_TIME_CALLBACK_PERIOD = 13;
		private static byte FUNCTION_GET_DATE_TIME_CALLBACK_PERIOD = 14;
		private static byte FUNCTION_SET_MOTION_CALLBACK_PERIOD = 15;
		private static byte FUNCTION_GET_MOTION_CALLBACK_PERIOD = 16;
		private static byte CALLBACK_COORDINATES = 17;
		private static byte CALLBACK_STATUS = 18;
		private static byte CALLBACK_ALTITUDE = 19;
		private static byte CALLBACK_MOTION = 20;
		private static byte CALLBACK_DATE_TIME = 21;


		/// <summary>
		///  This callback is triggered periodically with the period that is set by
		///  <see cref="Tinkerforge.BrickletGPS.SetCoordinatesCallbackPeriod"/>. The parameters are the same
		///  as for <see cref="Tinkerforge.BrickletGPS.GetCoordinates"/>.
		///  
		///  <see cref="Tinkerforge.BrickletGPS.Coordinates"/> is only triggered if the coordinates changed since the
		///  last triggering.
		/// </summary>
		public delegate void Coordinates(uint latitude, char ns, uint longitude, char ew, ushort pdop, ushort hdop, ushort vdop, ushort epe);

		/// <summary>
		///  This callback is triggered periodically with the period that is set by
		///  <see cref="Tinkerforge.BrickletGPS.SetStatusCallbackPeriod"/>. The parameters are the same
		///  as for <see cref="Tinkerforge.BrickletGPS.GetStatus"/>.
		///  
		///  <see cref="Tinkerforge.BrickletGPS.Status"/> is only triggered if the status changed since the
		///  last triggering.
		/// </summary>
		public delegate void Status(byte fix, byte satellitesView, byte satellitesUsed);

		/// <summary>
		///  This callback is triggered periodically with the period that is set by
		///  <see cref="Tinkerforge.BrickletGPS.SetAltitudeCallbackPeriod"/>. The parameters are the same
		///  as for <see cref="Tinkerforge.BrickletGPS.GetAltitude"/>.
		///  
		///  <see cref="Tinkerforge.BrickletGPS.Altitude"/> is only triggered if the altitude changed since the
		///  last triggering.
		/// </summary>
		public delegate void Altitude(uint altitude, uint geoidalSeparation);

		/// <summary>
		///  This callback is triggered periodically with the period that is set by
		///  <see cref="Tinkerforge.BrickletGPS.SetMotionCallbackPeriod"/>. The parameters are the same
		///  as for <see cref="Tinkerforge.BrickletGPS.GetMotion"/>.
		///  
		///  <see cref="Tinkerforge.BrickletGPS.Motion"/> is only triggered if the motion changed since the
		///  last triggering.
		/// </summary>
		public delegate void Motion(uint course, uint speed);

		/// <summary>
		///  This callback is triggered periodically with the period that is set by
		///  <see cref="Tinkerforge.BrickletGPS.SetDateTimeCallbackPeriod"/>. The parameters are the same
		///  as for <see cref="Tinkerforge.BrickletGPS.GetDateTime"/>.
		///  
		///  <see cref="Tinkerforge.BrickletGPS.DateTime"/> is only triggered if the date or time changed since the
		///  last triggering.
		/// </summary>
		public delegate void DateTime(uint date, uint time);

		/// <summary>
		///  Creates an object with the unique device ID <c>uid</c>. This object can
		///  then be added to the IP connection.
		/// </summary>
		public BrickletGPS(string uid) : base(uid) 
		{
			this.expectedName = "GPS Bricklet";
			this.bindingVersion[0] = 1;
			this.bindingVersion[1] = 0;
			this.bindingVersion[2] = 0;
			messageCallbacks[CALLBACK_COORDINATES] = new MessageCallback(CallbackCoordinates);
			messageCallbacks[CALLBACK_STATUS] = new MessageCallback(CallbackStatus);
			messageCallbacks[CALLBACK_ALTITUDE] = new MessageCallback(CallbackAltitude);
			messageCallbacks[CALLBACK_MOTION] = new MessageCallback(CallbackMotion);
			messageCallbacks[CALLBACK_DATE_TIME] = new MessageCallback(CallbackDateTime);
		}

		/// <summary>
		///  Returns the GPS coordinates. Latitude and longitude are given in the
		///  DD.dddddd° format. The parameter ns and ew are the cardinal directions for
		///  latitude and logitude. Possible values for ns and ew are 'N', 'S', 'E'
		///  and 'W' (north, south, east and west).
		///  
		///  PDOP, HDOP and VDOP are the dilution of precision (DOP) values. They specify
		///  the additional multiplicative effect of GPS satellite geometry on GPS 
		///  precision. See 
		///  `here &lt;http://en.wikipedia.org/wiki/Dilution_of_precision_(GPS)&gt;`__
		///  for more information. The values are give in hundreths.
		///  
		///  EPE is the "Estimated Position Error". The EPE is given in cm. This is not the
		///  absolute maximum error, it is the error with a specific confidence. See
		///  `here &lt;http://www.nps.gov/gis/gps/WhatisEPE.html&gt;`__ for more information.
		/// </summary>
		public void GetCoordinates(out uint latitude, out char ns, out uint longitude, out char ew, out ushort pdop, out ushort hdop, out ushort vdop, out ushort epe)
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_COORDINATES, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_COORDINATES, out response);

			latitude = LEConverter.UIntFrom(4, response);
			ns = LEConverter.CharFrom(8, response);
			longitude = LEConverter.UIntFrom(9, response);
			ew = LEConverter.CharFrom(13, response);
			pdop = LEConverter.UShortFrom(14, response);
			hdop = LEConverter.UShortFrom(16, response);
			vdop = LEConverter.UShortFrom(18, response);
			epe = LEConverter.UShortFrom(20, response);
		}

		/// <summary>
		///  Returns the current fix status, the number of satellites that are in view and
		///  the number of satellites that are currently used.
		///  
		///  Possible fix status values can be:
		///  
		///  <code>
		///   "Value", "Description"
		///  
		///   "1", "No Fix"
		///   "2", "2D Fix"
		///   "3", "3D Fix"
		///  </code>
		/// </summary>
		public void GetStatus(out byte fix, out byte satellitesView, out byte satellitesUsed)
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_STATUS, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_STATUS, out response);

			fix = LEConverter.ByteFrom(4, response);
			satellitesView = LEConverter.ByteFrom(5, response);
			satellitesUsed = LEConverter.ByteFrom(6, response);
		}

		/// <summary>
		///  Returns the current altitude and corresponding geoidal separation.
		///  
		///  Both values are given in cm.
		/// </summary>
		public void GetAltitude(out uint altitude, out uint geoidalSeparation)
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_ALTITUDE, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_ALTITUDE, out response);

			altitude = LEConverter.UIntFrom(4, response);
			geoidalSeparation = LEConverter.UIntFrom(8, response);
		}

		/// <summary>
		///  Returns the current course and speed. Course is given in hundreths degree
		///  and speed is given in hundreths km/h.
		///  
		///  Please note that this only returns useful values if an actual movement
		///  is present.
		/// </summary>
		public void GetMotion(out uint course, out uint speed)
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_MOTION, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_MOTION, out response);

			course = LEConverter.UIntFrom(4, response);
			speed = LEConverter.UIntFrom(8, response);
		}

		/// <summary>
		///  Returns the current date and time. The date is
		///  given in the format ddmmyy and the time is given
		///  in the format hhmmss.sss
		/// </summary>
		public void GetDateTime(out uint date, out uint time)
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_DATE_TIME, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_DATE_TIME, out response);

			date = LEConverter.UIntFrom(4, response);
			time = LEConverter.UIntFrom(8, response);
		}

		/// <summary>
		///  Restarts the GPS Bricklet, the following restart types are available:
		///  
		///  <code>
		///   "Value", "Description"
		///  
		///   "0", "Hot start (use all available data in the NV store)"
		///   "1", "Warm start (don't use ephemeris at restart)"
		///   "2", "Cold start (don't use time, position, almanacs and ephemeris at restart)"
		///   "3", "Factory reset (clear all system/user configurations at restart)"
		///  </code>
		/// </summary>
		public void Restart(byte restartType)
		{
			byte[] data_ = new byte[5];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_RESTART, 1, data_);
			LEConverter.To((ushort)5, 2, data_);
			LEConverter.To(restartType, 4, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Sets the period in ms with which the <see cref="Tinkerforge.BrickletGPS.Coordinates"/> callback is triggered
		///  periodically. A value of 0 turns the callback off.
		///  
		///  <see cref="Tinkerforge.BrickletGPS.Coordinates"/> is only triggered if the coordinates changed since the
		///  last triggering.
		///  
		///  The default value is 0.
		/// </summary>
		public void SetCoordinatesCallbackPeriod(uint period)
		{
			byte[] data_ = new byte[8];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_COORDINATES_CALLBACK_PERIOD, 1, data_);
			LEConverter.To((ushort)8, 2, data_);
			LEConverter.To(period, 4, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns the period as set by <see cref="Tinkerforge.BrickletGPS.SetCoordinatesCallbackPeriod"/>.
		/// </summary>
		public uint GetCoordinatesCallbackPeriod()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_COORDINATES_CALLBACK_PERIOD, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_COORDINATES_CALLBACK_PERIOD, out response);

			return LEConverter.UIntFrom(4, response);
		}

		/// <summary>
		///  Sets the period in ms with which the <see cref="Tinkerforge.BrickletGPS.Status"/> callback is triggered
		///  periodically. A value of 0 turns the callback off.
		///  
		///  <see cref="Tinkerforge.BrickletGPS.Status"/> is only triggered if the status changed since the
		///  last triggering.
		///  
		///  The default value is 0.
		/// </summary>
		public void SetStatusCallbackPeriod(uint period)
		{
			byte[] data_ = new byte[8];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_STATUS_CALLBACK_PERIOD, 1, data_);
			LEConverter.To((ushort)8, 2, data_);
			LEConverter.To(period, 4, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns the period as set by <see cref="Tinkerforge.BrickletGPS.SetStatusCallbackPeriod"/>.
		/// </summary>
		public uint GetStatusCallbackPeriod()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_STATUS_CALLBACK_PERIOD, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_STATUS_CALLBACK_PERIOD, out response);

			return LEConverter.UIntFrom(4, response);
		}

		/// <summary>
		///  Sets the period in ms with which the <see cref="Tinkerforge.BrickletGPS.Altitude"/> callback is triggered
		///  periodically. A value of 0 turns the callback off.
		///  
		///  <see cref="Tinkerforge.BrickletGPS.Altitude"/> is only triggered if the altitude changed since the
		///  last triggering.
		///  
		///  The default value is 0.
		/// </summary>
		public void SetAltitudeCallbackPeriod(uint period)
		{
			byte[] data_ = new byte[8];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_ALTITUDE_CALLBACK_PERIOD, 1, data_);
			LEConverter.To((ushort)8, 2, data_);
			LEConverter.To(period, 4, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns the period as set by <see cref="Tinkerforge.BrickletGPS.SetAltitudeCallbackPeriod"/>.
		/// </summary>
		public uint GetAltitudeCallbackPeriod()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_ALTITUDE_CALLBACK_PERIOD, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_ALTITUDE_CALLBACK_PERIOD, out response);

			return LEConverter.UIntFrom(4, response);
		}

		/// <summary>
		///  Sets the period in ms with which the <see cref="Tinkerforge.BrickletGPS.DateTime"/> callback is triggered
		///  periodically. A value of 0 turns the callback off.
		///  
		///  <see cref="Tinkerforge.BrickletGPS.DateTime"/> is only triggered if the date or time changed since the
		///  last triggering.
		///  
		///  The default value is 0.
		/// </summary>
		public void SetDateTimeCallbackPeriod(uint period)
		{
			byte[] data_ = new byte[8];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_DATE_TIME_CALLBACK_PERIOD, 1, data_);
			LEConverter.To((ushort)8, 2, data_);
			LEConverter.To(period, 4, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns the period as set by <see cref="Tinkerforge.BrickletGPS.SetDateTimeCallbackPeriod"/>.
		/// </summary>
		public uint GetDateTimeCallbackPeriod()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_DATE_TIME_CALLBACK_PERIOD, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_DATE_TIME_CALLBACK_PERIOD, out response);

			return LEConverter.UIntFrom(4, response);
		}

		/// <summary>
		///  Sets the period in ms with which the <see cref="Tinkerforge.BrickletGPS.Motion"/> callback is triggered
		///  periodically. A value of 0 turns the callback off.
		///  
		///  <see cref="Tinkerforge.BrickletGPS.Motion"/> is only triggered if the motion changed since the
		///  last triggering.
		///  
		///  The default value is 0.
		/// </summary>
		public void SetMotionCallbackPeriod(uint period)
		{
			byte[] data_ = new byte[8];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_MOTION_CALLBACK_PERIOD, 1, data_);
			LEConverter.To((ushort)8, 2, data_);
			LEConverter.To(period, 4, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns the period as set by <see cref="Tinkerforge.BrickletGPS.SetMotionCallbackPeriod"/>.
		/// </summary>
		public uint GetMotionCallbackPeriod()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_MOTION_CALLBACK_PERIOD, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_MOTION_CALLBACK_PERIOD, out response);

			return LEConverter.UIntFrom(4, response);
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetCoordinatesCallbackPeriod(out uint period)
		{
			period = GetCoordinatesCallbackPeriod();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetStatusCallbackPeriod(out uint period)
		{
			period = GetStatusCallbackPeriod();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetAltitudeCallbackPeriod(out uint period)
		{
			period = GetAltitudeCallbackPeriod();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetDateTimeCallbackPeriod(out uint period)
		{
			period = GetDateTimeCallbackPeriod();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetMotionCallbackPeriod(out uint period)
		{
			period = GetMotionCallbackPeriod();
		}

		internal int CallbackCoordinates(byte[] data_)
		{
			uint latitude = LEConverter.UIntFrom(4, data_);
			char ns = LEConverter.CharFrom(8, data_);
			uint longitude = LEConverter.UIntFrom(9, data_);
			char ew = LEConverter.CharFrom(13, data_);
			ushort pdop = LEConverter.UShortFrom(14, data_);
			ushort hdop = LEConverter.UShortFrom(16, data_);
			ushort vdop = LEConverter.UShortFrom(18, data_);
			ushort epe = LEConverter.UShortFrom(20, data_);

			((Coordinates)callbacks[CALLBACK_COORDINATES])(latitude, ns, longitude, ew, pdop, hdop, vdop, epe);
			return 22;
		}

		internal int CallbackStatus(byte[] data_)
		{
			byte fix = LEConverter.ByteFrom(4, data_);
			byte satellitesView = LEConverter.ByteFrom(5, data_);
			byte satellitesUsed = LEConverter.ByteFrom(6, data_);

			((Status)callbacks[CALLBACK_STATUS])(fix, satellitesView, satellitesUsed);
			return 7;
		}

		internal int CallbackAltitude(byte[] data_)
		{
			uint altitude = LEConverter.UIntFrom(4, data_);
			uint geoidalSeparation = LEConverter.UIntFrom(8, data_);

			((Altitude)callbacks[CALLBACK_ALTITUDE])(altitude, geoidalSeparation);
			return 12;
		}

		internal int CallbackMotion(byte[] data_)
		{
			uint course = LEConverter.UIntFrom(4, data_);
			uint speed = LEConverter.UIntFrom(8, data_);

			((Motion)callbacks[CALLBACK_MOTION])(course, speed);
			return 12;
		}

		internal int CallbackDateTime(byte[] data_)
		{
			uint date = LEConverter.UIntFrom(4, data_);
			uint time = LEConverter.UIntFrom(8, data_);

			((DateTime)callbacks[CALLBACK_DATE_TIME])(date, time);
			return 12;
		}

		/// <summary>
		///  Registers a callback function.
		/// </summary>
		public void RegisterCallback(System.Delegate d)
		{
			if(d.GetType() == typeof(Coordinates))
			{
				callbacks[CALLBACK_COORDINATES] = d;
			}
			else if(d.GetType() == typeof(Status))
			{
				callbacks[CALLBACK_STATUS] = d;
			}
			else if(d.GetType() == typeof(Altitude))
			{
				callbacks[CALLBACK_ALTITUDE] = d;
			}
			else if(d.GetType() == typeof(Motion))
			{
				callbacks[CALLBACK_MOTION] = d;
			}
			else if(d.GetType() == typeof(DateTime))
			{
				callbacks[CALLBACK_DATE_TIME] = d;
			}
		}
	}
}
